#
# Copyright (c) 2011 by Michael Thomas Greer
# See the file "license.terms" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#
# Copyright 2016 Jochen Fabricius
# Distribution and modification is encouraged

package provide tk_dia 1.1

package require Tk 8.6

#-----------------------------------------------------------------------------
namespace eval ::tk::tk_dia:: {

  proc refocus args {
    # Used to force the modal dialog to have focus after
    # evaluating any of the user command option scripts.
    # (Since the script could do just about anything...)
    focus -force [lindex $args 0]
    }

  }

#-----------------------------------------------------------------------------
proc ::tk::tk_dia args {
  if {[llength $args] % 2} {
    return -code error {wrong # args: must be "tk_dia ?option value ...?"}
    }

  array set options {
   #-default         {this must not exist unless specified by the user}
    -fractionx       .50
    -fractiony       .425
    -height          10
    -helpcommand     {}
    -invalidcommand  {}
    -parent          {}
    -prompt          {}
    -show            {}
    -state           normal
   #-title           {this must not exist unless specified by the user}
    -validatecommand {}
    -values          {}
    -width           0
    -grab             yes
    -buttons         {{Ok {Return KP_Enter}} {Cancel {Escape Destroy}}}
    }
  array set options $args
  foreach {name abbreviations} {
    -helpcommand     {-hcmd -helpcmd}
    -invalidcommand  {-icmd -invalidcmd}
    -validatecommand {-vcmd -validatecmd}
    } {
    foreach abbreviation $abbreviations {
      if {[info exists options($abbreviation)]} {
        set options($name) $options($abbreviation)
    } } }

  # Validate options .........................................................
  if {($options(-parent) ne {}) && ![winfo exists $options(-parent)]} {
    error "bad window path name \"$options(-parent)\""
    }
  foreach opt {fractionx fractiony} {
    set v $options(-$opt)
    if {![string is double -strict $v] || ($v < 0) || ($v > 1)} {
      error "expected floating-point $opt in \[0.0, 1.0\] but got \"$v\""
    } }
  foreach opt {height width} {
    if {![string is integer -strict $options(-$opt)]} {
      error "expected integer $opt but got \"$options(-$opt)\""
    } }
  if {$options(-state) ni {normal readonly}} {
    error "bad state \"$options(-state)\": must be normal or readonly"
    }

  if {![info exist options(-title)]} {  # we must have a -title
    if {$options(-parent) ne {}} \
      then { set options(-title) [wm title $options(-parent)] } \
      else { set options(-title) [wm title .]                 }
    }
  if {$options(-show) ne {}} {          # -show beats -values
    set options(-values) {}
    }

  # Create and populate the dialog window ....................................
  set w [string map {.. .} $options(-parent).[clock microseconds]]

  toplevel $w -relief flat -class TkGetStringDialog
  variable ::tk::$w.buttonpressed
  #variable ::tk::$w.refocus {}
  #trace add variable ::tk::$w.refocus write [list ::tk::tk_dia::refocus $w.entry]

  wm title     $w $options(-title)
  wm iconname  $w $options(-title)
  wm protocol  $w WM_DELETE_WINDOW [list set ::tk::$w.buttonpressed cancel]
  wm transient $w [winfo toplevel [winfo parent $w]]

  set prev_focus   [focus -displayof $w]
  set prev_grab    [grab current $w]

  # (The prompt message, if any)
  if {$options(-prompt) ne {}} {
    ttk::label $w.prompt -text $options(-prompt)
    pack $w.prompt -side top -expand yes -fill x
    }

  # (Command options)
  foreach cmd {-helpcommand -invalidcommand -validatecommand} {
    if {[llength $options($cmd)] != 0} {
      set options($cmd) "set ::tk::$w.refocus \[ $options($cmd) \]"
    } }

  # (Buttons)
  ttk::frame  $w.buttons
  set i 0
  foreach btn_def $options(-buttons) {
    lassign $btn_def button bindings
    set btn_name [string map {{ } _} [string tolower $button]]
    ttk::button $w.buttons.$btn_name     -text $button     -command [list set ::tk::$w.buttonpressed $i]
    pack $w.buttons.$btn_name     -side left -expand yes -fill x
    foreach b $bindings {
      bind $w <$b> [list set ::tk::$w.buttonpressed $button]
    }
    incr i
  }
  pack $w.buttons -expand yes -fill x

  # (Global bindings)
  bind $w <F1>       $options(-helpcommand)
  bind $w <Help>     $options(-helpcommand)

  # Properly position it on the display ......................................
  # See "Total Window Geometry" http://wiki.tcl.tk/11291
  wm withdraw $w
  update idletasks
  #focus -force $w.entry
  if {$options(-parent) eq {}} \
    then {
      # (Position on the user's screen/vroot)
      lassign [split [winfo geometry $w] +] foo dtop dleft
      set dw [expr {[winfo rootx $w] - $dleft}]
      set dh [expr {[winfo rooty $w] - $dtop }]
      set x [expr {round( ([winfo vrootwidth  $w] - [winfo reqwidth  $w] - $dw) * $options(-fractionx) )}]
      set y [expr {round( ([winfo vrootheight $w] - [winfo reqheight $w] - $dh) * $options(-fractiony) )}]
      } \
    else {
      # (Position on the parent widget)
      set p $options(-parent)
      set x [expr {round( (([winfo width  $p] - [winfo reqwidth  $w]) * $options(-fractionx)) + [winfo x $p] )}]
      set y [expr {round( (([winfo height $p] - [winfo reqheight $w]) * $options(-fractiony)) + [winfo y $p] )}]
      }
  incr x [expr {[winfo vrootx $w]/2}]
  incr y [expr {[winfo vrooty $w]/2}]
  wm geometry $w +$x+$y
  wm deiconify $w
  wm resizable $w 0 0
  if {$options(-grab)} {grab $w}

  # Run the dialog ...........................................................
  tkwait variable ::tk::$w.buttonpressed

  set result [set ::tk::$w.buttonpressed]

  # Clean up .................................................................
  if {$options(-grab)} {grab release $w}
  destroy $w
  focus -force $prev_focus
  if {$prev_grab ne {}} { grab $prev_grab }
  update idletasks

  unset ::tk::$w.buttonpressed

  return $result
  }

namespace eval ::tk:: { namespace export tk_dia }
namespace import ::tk::tk_dia
